package MT::Plugin::SKR::SortSearchResultsAcrossBlog;
########################################################################
#   SortSearchResultsAcrossBlog
#           Copyright (c) 2007 Skyarc System Co.,Ltd.
#           @see http://www.skyarc.co.jp/engineerblog/entry/3636.html
########################################################################
use strict;

use vars qw( $MYNAME $VERSION );
$MYNAME = 'SortSearchResultsAcrossBlog';
$VERSION = '1.11';

use base qw( MT::Plugin );
my $plugin = new MT::Plugin ({
        name => $MYNAME,
        version => $VERSION,
        author_name => q{ SKYARC System Co.,Ltd. },
        author_link => q{ http://www.skyarc.co.jp },
        doc_link => qw{ http://www.skyarc.co.jp/engineerblog/entry/3636.html },
        description => <<HTMLHEREDOC,
Re-sort the search results across all including blogs.
HTMLHEREDOC
});
MT->add_plugin( $plugin );

sub instance { $plugin }



### Re-sort the search results across all including blogs.
use MT::Template::Context;
MT::Template::Context->add_container_tag (SortSearchResultsAcrossBlog => \&sort_results);
sub sort_results {
    my ($ctx, $args, $cond) = @_;

    # This extension is supplied for search result template.
    my $results = $ctx->stash ('results');
    if ($results) {
        my $app = MT->instance;
        my $q = $app->param;
        my @res = @{ $results };

        # Limitation by created date
        if (my $date = $q->param('DateLimit') || $args->{'date_limit'}) {
            my ($fr, $span, $to) = $date =~ m/^\s*(\d+)?\s*(-)?\s*(\d+)?\s*$/;

            $ctx->{__stash}->{__PACKAGE__}->{from} = $fr;
            $ctx->{__stash}->{__PACKAGE__}->{span} = $span;
            $ctx->{__stash}->{__PACKAGE__}->{to} = $to;

            if ($span) {
                $fr = substr $fr. '0' x 14, 0, 14 if $fr;
                $to = substr $to. '9' x 14, 0, 14 if $to;
                @res = grep {
                    ($fr ? $fr <= $_->{entry}->created_on : 1) && ($to ? $_->{entry}->created_on <= $to : 1)
                } @res;
            } elsif ($fr) {
                @res = grep {
                    $_->{entry}->created_on =~ /^$fr/
                } @res;
            }
        }

        # Sort again across all blogs. In default, by created_on in descending
        my $col = $app->{searchparam}{'SearchSortBy'} || 'created_on';
        my $order = $app->{searchparam}{'ResultDisplay'} || 'ascend';
        @res = $order eq 'ascend'
                ? sort { $a->{entry}->$col() cmp $b->{entry}->$col() } @res
                : sort { $b->{entry}->$col() cmp $a->{entry}->$col() } @res;
        # Reset Header/Footer flag
        map {
            $_->{blogheader} = $_->{blogfooter} = 0;
        } @res;

        $results = \@res;
    }

    # Replace with modified results as local value
    local $ctx->{__stash}->{results} = $results;

    # Pass throught
    my $builder = $ctx->stash ('builder');
    my $tokens = $ctx->stash ('tokens');
    defined (my $out = $builder->build ($ctx, $tokens, $cond))
        or return $ctx->error ($builder->errstr);
    $out;
}

### Tag: MTSearchDateLimitFrom/To, Conditional Tag: MTSearchIfDateLimit
MT::Template::Context->add_tag (
    SearchDateLimitFrom => sub { $_[0]->{__stash}->{__PACKAGE__}->{from} ||= ''; });
MT::Template::Context->add_tag (
    SearchDateLimitTo => sub { $_[0]->{__stash}->{__PACKAGE__}->{to} ||= ''; });
MT::Template::Context->add_conditional_tag (
    SearchIfDateLimit => sub { defined $_[0]->{__stash}->{__PACKAGE__}->{span}; });

1;
__END__
########################################################################
# '07/10/30 1.11    update: Localize the modified results in the context
# '07/10/17 1.10    update: Add tags; $MTSearchDateLimitFrom/To$, $MTSearchIfDateLimit$
# '07/10/10 1.01    bugfix: DateLimit parameter doesn't work
#                   bugfix: Date spanning dowsn't work specified by DateLimit parameter
#                   check: MTE153 OK by tsushima
# '07/10/09 1.00    create: by pirolix
#                   check: MT334 OK by pirolix
